
# q-expansion of newform 45.6.a.e, downloaded from the LMFDB on 18 January 2025.

# We generate the q-expansion using the Hecke eigenvalues a_p at the primes.
# Each a_p is given as a linear combination
# of the following basis for the coefficient ring.

def make_data():

    from sage.all import prod, floor, prime_powers, gcd, QQ, primes_first_n, next_prime, RR

    def discrete_log(elts, gens, mod):
        # algorithm 2.2, page 16 of https://arxiv.org/abs/0903.2785
        def table_gens(gens, mod):
            T = [1]
            n = len(gens)
            r = [None]*n
            s = [None]*n
            for i in range(n):
                beta = gens[i]
                r[i] = 1
                N = len(T)
                while beta not in T:
                    for Tj in T[:N]:
                        T.append((beta*Tj) % mod)
                    beta = (beta*gens[i]) % mod
                    r[i] += 1
                s[i] = T.index(beta)
            return T, r, s
        T, r, s = table_gens(gens, mod)
        n = len(gens)
        N = [ prod(r[:j]) for j in range(n) ]
        Z = lambda s: [ (floor(s/N[j]) % r[j]) for j in range(n)]
        return [Z(T.index(elt % mod)) for elt in elts]
    def extend_multiplicatively(an):
        for pp in prime_powers(len(an)-1):
            for k in range(1, (len(an) - 1)//pp + 1):
                if gcd(k, pp) == 1:
                    an[pp*k] = an[pp]*an[k]
    from sage.all import PolynomialRing, NumberField
    R = PolynomialRing(QQ, "x")
    f = R(poly_data)
    K = NumberField(f, "a")
    betas = [K.gens()[0]**i for i in range(len(poly_data))]
    convert_elt_to_field = lambda elt: sum(c*beta for c, beta in zip(elt, betas))
    # convert aps to K elements
    primes = primes_first_n(len(aps_data))
    good_primes = [p for p in primes if not p.divides(level)]
    aps = map(convert_elt_to_field, aps_data)
    if not hecke_ring_character_values:
        # trivial character
        char_values = dict(zip(good_primes, [1]*len(good_primes)))
    else:
        gens = [elt[0] for elt in hecke_ring_character_values]
        gens_values = [convert_elt_to_field(elt[1]) for elt in hecke_ring_character_values]
        char_values = dict([(
            p,prod(g**k for g, k in zip(gens_values, elt)))
            for p, elt in zip(good_primes, discrete_log(good_primes, gens, level))
            ])
    an_list_bound = next_prime(primes[-1])
    an = [0]*an_list_bound
    an[1] = 1
    
    from sage.all import PowerSeriesRing
    PS = PowerSeriesRing(K, "q")
    for p, ap in zip(primes, aps):
        if p.divides(level):
            euler_factor = [1, -ap]
        else:
            euler_factor = [1, -ap, p**(weight - 1) * char_values[p]]
        k = RR(an_list_bound).log(p).floor() + 1
        foo = (1/PS(euler_factor)).padded_list(k)
        for i in range(1, k):
            an[p**i] = foo[i]
    extend_multiplicatively(an)
    return PS(an)
level = 45
weight = 6
poly_data = [-102, -1, 1]

# The basis for the coefficient ring is just the power basis
# in the root of the defining polynomial above.
hecke_ring_character_values = None
aps_data = [[0, 1], [0, 0], [-25, 0], [-64, 16], [-108, -32], [446, -16], [-978, -80], [836, -208], [1632, -48], [-942, -64], [1424, -176], [3926, 816], [4086, -544], [-8188, -64], [10296, -1232], [6042, 2272], [-1164, 3232], [9326, 1568], [-5812, -1280], [18888, 3200], [29258, 608], [51920, 3760], [63060, -4032], [-48186, -7392], [-6142, -12480], [-34518, 5568], [-36208, -9264], [106956, -3552], [-40354, -2688], [8430, -2192], [-13672, -22000], [96924, -4608], [-202890, -4464], [176444, -14832], [295002, -10816], [175592, 35600], [-393490, -14256], [534044, -12480], [-545856, 12624], [-14430, 38208], [-55140, 58240], [-330586, 29184], [300432, 15488], [205730, -48288], [499338, -35808], [54728, 16016], [328724, -65136], [303656, 41904], [-325356, -84736], [-622186, -56672], [-193482, -33456], [950064, -8672], [884018, 40832], [1078308, -16320], [-1142754, 79728], [462576, -15760], [955074, -114752], [-233632, -42928], [-1538938, 36688], [-2301690, -10656], [860276, 5824], [393546, -4320], [988604, 63392], [741432, -101472], [653594, 220320], [598386, 71648], [-3413908, -4400], [-2125678, -3232], [-1238628, 227680], [-154258, -26656], [126654, -60144], [1505016, -104448], [-675928, 353008], [-1910938, -99184], [195020, -13840], [-2046984, 122160], [373866, 287232], [-1185346, 203312], [-1416786, -406848], [1783034, 173568], [-3530196, -132352], [-772810, -43744], [-294624, 222336], [1871954, -364960], [2873720, -414640], [-2240436, -292224], [4843902, 74944], [25706, -151744], [-3540702, 307200], [-1723048, 13456], [1068036, -59872], [-6096000, 168960], [1126208, -309840], [2415924, 204512], [795236, 292112], [-410016, 243056], [246066, -439808], [3577686, -65344], [696932, 413216], [3293294, 817024], [1411436, -1020544], [1915362, 397440], [-10567500, -182112], [6189414, -248832], [790748, 500112], [1241858, -155520], [1821516, 306528], [-567282, 628784], [-2458008, 137984], [6483578, -747648], [-6157192, 460400], [9743030, -541648], [-6889290, 161616], [-6268036, 127248], [-13262968, -366160], [-2032194, -1572384], [5718332, -126464], [10787232, -942320], [379746, -526720], [-4102020, 947680], [-4050778, 1478240], [609218, -141152], [2935338, -1319808], [1160316, 522720], [10087508, 163472], [-5752686, 1858112], [9759926, -60448], [-4087728, -37536], [-4558000, 190704], [-8686738, 179856], [10822004, -1748272], [5436528, -244976], [15132128, -1050928], [2199590, 1414544], [2040870, 732864], [1761218, 649536], [-420534, 2339040], [13589564, 702752], [-13018734, -1225792], [-10663050, -2112160], [19557932, 875920], [5866170, -2270976], [-12099520, 2164752], [28257852, 374080], [27278510, 94720], [-7473480, 1864640], [9499814, -536720], [-4463514, 3116848], [18118220, 997040], [-19323912, -544816], [-3900994, -2675504], [6680910, -1309856], [10407692, 479616], [22095696, 1703568], [8342324, 2358112], [-42912, -2711840], [-3285640, 1232720], [15646494, 2964608], [-15950038, 1554688], [6959106, -1858624], [9367620, -1888224], [23339334, -2822384], [-28398592, -337680], [4024116, -276864], [-29196498, -453200], [-32914848, 507792], [25128272, 3408240], [-10267210, -352496]]
